@@ -5,7 +5,7 @@
  * This file contains the implementation of the main KBiff
  * widget
  *
- * $Id: kbiff.cpp,v 1.3 2001/05/21 21:54:38 gran Exp $
+ * $Id: kbiff.cpp,v 1.5 2001/10/22 22:35:29 gran Exp $
  */
 #include "kbiff.h"
 #include <qmovie.h>
@@ -97,7 +97,7 @@ void KBiff::processSetup(const KBiffSetup* setup, bool run)
     playSoundPath  = setup->getPlaySoundPath();
     notify         = setup->getNotify();
     dostatus       = setup->getStatus();
-    	
+
     // if we aren't going the status route, we should at least
     // provide a tooltip!
     if (dostatus == false)
@@ -464,7 +464,7 @@ void KBiff::haveNewMail(const int num, const QString& the_mailbox)
         // make sure the command exists
         if (!runCommandPath.isEmpty())
         {
-            executeCommand(runCommandPath);
+            executeCommand(replaceCommandArgs(runCommandPath));
         }
     }
 
@@ -483,6 +483,56 @@ void KBiff::haveNewMail(const int num, const QString& the_mailbox)
     }
 }
 
+QString KBiff::getURLWithNewMail()
+{
+    KBiffMonitor *monitor;
+    for (monitor = monitorList.first();
+            monitor != 0;
+            monitor = monitorList.next())
+    {
+        if(monitor->getMailState() == NewMail)
+            return monitor->getMailbox();
+    }
+
+    return monitorList.first()->getMailbox();
+}
+
+QString KBiff::getMailBoxWithNewMail()
+{
+    QString url(getURLWithNewMail());
+    
+    int slashPos = url.find('/');
+    if(slashPos == -1)
+      return url.mid(slashPos + 1);
+    else
+      return url.mid(url.find(':') + 1);
+}
+
+QString KBiff::replaceCommandArgs(QString cmdStr)
+{
+    bool expand = false;
+    for(unsigned int i = 0; i < cmdStr.length(); i++)
+    {
+        if(expand)
+        {
+            expand = false;
+            if(cmdStr[i] == 'm')
+                cmdStr.replace(i - 1, 2, getMailBoxWithNewMail());
+            else if(cmdStr[i] == 'u')
+                cmdStr.replace(i - 1, 2, getURLWithNewMail());
+            else if(cmdStr[i] == '%')
+                cmdStr.replace(i - 1, 2, "%");
+
+            continue;
+        }
+
+        if(cmdStr[i] == '%')
+            expand = true;
+    }
+
+    return cmdStr;
+}
+
 void KBiff::dock()
 {
     // destroy the old window
@@ -685,7 +735,7 @@ void KBiff::slotLaunchFetchClient(const QString& fetchClient)
 void KBiff::slotLaunchMailClient()
 {
     if (!mailClient.isEmpty())
-        executeCommand(mailClient);
+        executeCommand(replaceCommandArgs(mailClient));
 }
 
 void KBiff::slotPlaySound(const QString& play_sound)
