@@ -5,7 +5,7 @@
  * This file contains the implementation of KBiffMonitor and
  * associated classes.
  *
- * $Id: kbiffmonitor.cpp,v 1.3 2001/05/21 21:54:38 gran Exp $
+ * $Id: kbiffmonitor.cpp,v 1.7 2001/10/23 19:52:18 gran Exp $
  */
 #include "kbiffmonitor.h"
 
@@ -27,6 +27,8 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
+#include <sys/types.h>
+#include <utime.h>
 
 #include <fcntl.h>
 #include <errno.h>
@@ -58,7 +60,8 @@ KBiffMonitor::KBiffMonitor()
       oldTimer(0),
       started(false),
       newCount(0),
-      oldCount(0),
+      oldCount(-1),
+      firstRun(false),
       key(""),
       simpleURL(""),
       protocol(""),
@@ -104,7 +107,6 @@ KBiffMonitor::~KBiffMonitor()
 
 void KBiffMonitor::readConfig()
 {
-    kdDebug() << "KBiffMonitor: reading saved state" << endl;
     KSimpleConfig *config = new KSimpleConfig(MAIL_STATE_FILE);
     config->setDollarExpansion(false);
 
@@ -114,7 +116,7 @@ void KBiffMonitor::readConfig()
 
     QStrList list;
 
-    mailState = (KBiffMailState)config->readNumEntry("mailState");
+    mailState = (KBiffMailState)config->readNumEntry("mailState", UnknownState);
     lastSize = config->readNumEntry("lastSize");
     config->readListEntry("lastRead", list);
     if (list.count()==6)
@@ -137,6 +139,9 @@ void KBiffMonitor::readConfig()
         uidlList.append( new QString(UIDL) );
     }
 
+    newCount = config->readNumEntry("newCount", 0);
+    oldCount = config->readNumEntry("oldCount", -1);
+
     delete config;
 }
 
@@ -162,20 +167,22 @@ void KBiffMonitor::saveConfig()
     config->writeEntry("lastRead",lastRead);
     config->writeEntry("lastModified",lastModified);
     config->writeEntry("uidlList",uidlist);
+    config->writeEntry("newCount", newCount);
+    config->writeEntry("oldCount", oldCount);
 
     delete config;
 }
 
 void KBiffMonitor::onStateChanged()
 {
-	kdDebug() << "state changed" << endl;
-	saveConfig();
+    saveConfig();
 }
 
 void KBiffMonitor::start()
 {
     readConfig();
     started  = true;
+    firstRun = true;
     oldTimer = startTimer(poll * 1000);
     emit(signal_checkMail());
 }
@@ -360,7 +367,6 @@ void KBiffMonitor::setMailboxIsRead()
     b_new_uidlList = false;
 
     determineState(OldMail);
-    kdDebug() << "Active state has been reset" << endl;
   }
 }
 
@@ -395,6 +401,8 @@ void KBiffMonitor::checkLocal()
 
     // check if we have new mail
     determineState(mbox.size(), mbox.lastRead(), mbox.lastModified());
+
+    firstRun = false;
 }
 
 void KBiffMonitor::checkMbox()
@@ -407,7 +415,8 @@ void KBiffMonitor::checkMbox()
         emit(signal_fetchMail(fetchCommand));
 
     // see if the state has changed
-    if ((mbox.lastModified() != lastModified) || (mbox.size() != lastSize))
+    if ((mbox.lastModified() != lastModified) || (mbox.size() != lastSize) ||
+        (mailState == UnknownState) || (oldCount == -1))
     {
         lastModified = mbox.lastModified();
         lastSize     = mbox.size();
@@ -416,6 +425,29 @@ void KBiffMonitor::checkMbox()
         // new messages has, too.
         newCount = mboxMessages();
 
+        // Set access time of the file to what it was. If we don't do
+        // this some (all?) MUAs think that the mail has already been
+        // read.
+        {
+            utimbuf buf;
+            // we use 'setTime_t' since it takes timezone into account
+            QDateTime unixEpoch;
+            unixEpoch.setTime_t(0);
+            // do a simple test for those systems with hosed timezones.  if
+            // the lastRead and calculated lastRead aren't the same, then
+            // something is wrong with the system
+            QDateTime calc;
+            calc.setTime_t(-mbox.lastRead().secsTo(unixEpoch));
+            if (mbox.lastRead() == calc)
+            {
+              // QDateTime::secsTo returns a negative value if the argument
+              // is before the time the object refers to.
+              buf.actime = -mbox.lastRead().secsTo(unixEpoch);
+              buf.modtime = -mbox.lastModified().secsTo(unixEpoch);
+              utime(QFile::encodeName(mailbox), &buf);
+            }
+        }
+
         // if there are any new messages, consider the state New
         if (newCount > 0)
             determineState(NewMail);
@@ -427,6 +459,14 @@ void KBiffMonitor::checkMbox()
                 determineState(OldMail);
         }
     }
+    else if (firstRun)
+    {
+        KBiffMailState state(mailState);
+        mailState = UnknownState;
+        determineState(state);
+    }
+
+    firstRun = false;
 
     // handle the NoMail case
     if ((mbox.size() == 0) || (oldCount == 0))
@@ -439,6 +479,8 @@ void KBiffMonitor::checkMbox()
 
 void KBiffMonitor::checkPop()
 {
+    firstRun = false;
+
     QString command;
 
     // connect to the server unless it is active already
@@ -499,6 +541,8 @@ void KBiffMonitor::checkPop()
 
 void KBiffMonitor::checkImap()
 {
+    firstRun = false;
+
     QString command;
     int seq = 1000;
     bool do_login = false;
@@ -535,8 +579,21 @@ void KBiffMonitor::checkImap()
         seq++;
     }
 
-    // this will quite nicely get us the number of new messages
-    command = QString().setNum(seq) + " STATUS " + mailbox + " (messages unseen)\r\n";
+    // reset the numbers from the last check
+    imap->resetNumbers();
+    // select the mailbox
+    command = QString().setNum(seq) + " EXAMINE " + mailbox + "\r\n";
+    if (imap->command(command, seq) == false)
+        return;
+    seq++;
+
+    // this gets the number of undeleted messages
+    command = QString().setNum(seq) + " SEARCH UNDELETED\r\n";
+    if (imap->command(command, seq) == false)
+        return;
+    seq++;
+    // this gets the number of undeleted _new_ messages
+    command = QString().setNum(seq) + " SEARCH UNDELETED UNSEEN\r\n";
     if (imap->command(command, seq) == false)
         return;
     seq++;
@@ -568,6 +625,8 @@ void KBiffMonitor::checkImap()
 
 void KBiffMonitor::checkMaildir()
 {
+    firstRun = false;
+
     // get the information about this local mailbox
     QDir mbox(mailbox);
 
@@ -629,6 +688,8 @@ void KBiffMonitor::checkMaildir()
 
 void KBiffMonitor::checkNntp()
 {
+    firstRun = false;
+
     QString command;
     bool do_login = false;
 
@@ -766,6 +827,8 @@ void KBiffMonitor::checkNntp()
  */
 void KBiffMonitor::checkMHdir()
 {
+    firstRun = false;
+
     // get the information about this local mailbox
     QDir mbox(mailbox);
     char the_buffer[MAXSTR];
@@ -1480,6 +1543,9 @@ KBiffImap::~KBiffImap()
 bool KBiffImap::command(const QString& line, unsigned int seq)
 {
     int len, match;
+    bool unseen = false;
+    QString messagesListString;
+    QStringList messagesList;
 
     if (writeLine(line) <= 0)
     {
@@ -1503,15 +1569,20 @@ bool KBiffImap::command(const QString& line, unsigned int seq)
         if ((response.find(bad) > -1) || (response.find(no) > -1))
             break;
 
-        // check the number of messages
-        QRegExp messages_re("MESSAGES [0-9]*");
-        if ((match = messages_re.match(response, 0, &len)) > -1)
-            messages = response.mid(match + 9, len - 9).toInt();
+        QRegExp unseen_re(" UNSEEN");
+        if (unseen_re.match(line) > -1)
+            unseen = true;
 
-        // check for new mail
-        QRegExp recent_re("UNSEEN [0-9]*");
-        if ((match = recent_re.match(response, 0, &len)) > -1)
-            newMessages = response.mid(match + 7, len - 7).toInt();
+        QRegExp messages_re("SEARCH [0-9 ]*");
+        if ((match = messages_re.match(response, 0, &len)) > -1)
+        {
+            messagesListString = response.mid(match + 7, len - 7);
+            messagesList = QStringList::split(' ', messagesListString);
+            if (unseen)
+                newMessages = messagesList.count();
+            else
+                messages = messagesList.count();
+        }
     }
 
     close();
@@ -1530,6 +1601,12 @@ QString KBiffImap::mungeUserPass(const QString& old_user)
     return new_user;
 }
 
+void KBiffImap::resetNumbers()
+{
+    messages = 0;
+    newMessages = 0;
+}
+
 ///////////////////////////////////////////////////////////////////////////
 // KBiffPop
 ///////////////////////////////////////////////////////////////////////////
@@ -1705,7 +1782,7 @@ static bool real_from(const QString& orig_buffer)
     /* <weekday> */
     found = 0;
     for (i = 0; day_name[i] != NULL; i++)
-        found = found || (strnicmp(day_name[i], buffer, 3) == 0);
+        found = found || (qstrnicmp(day_name[i], buffer, 3) == 0);
 
     if (!found)
         return false;
@@ -1715,7 +1792,7 @@ static bool real_from(const QString& orig_buffer)
     /* <month> */
     found = 0;
     for (i = 0; month_name[i] != NULL; i++)
-        found = found || (strnicmp(month_name[i], buffer, 3) == 0);
+        found = found || (qstrnicmp(month_name[i], buffer, 3) == 0);
     if (!found)
         return false;
 
@@ -1732,7 +1809,7 @@ static const char* compare_header(const char* header, const char* field)
 {
     int len = strlen(field);
 
-    if (strnicmp(header, field, len))
+    if (qstrnicmp(header, field, len))
         return NULL;
 
     header += len;
