@@ -5,7 +5,7 @@
  * This file contains the implementation of the setup dialog
  * class for KBiff.
  *
- * $Id: setupdlg.cpp,v 1.4 2001/05/21 21:54:38 gran Exp $
+ * $Id: setupdlg.cpp,v 1.6 2001/10/23 19:52:18 gran Exp $
  */
 #include "setupdlg.h"
 
@@ -1169,6 +1169,7 @@ KBiffMailboxTab::KBiffMailboxTab(const QString& profile, QWidget *parent)
   QWhatsThis::add(checkFetchCommand, whatsthis);
   QWhatsThis::add(editFetchCommand, whatsthis);
   QWhatsThis::add(buttonBrowseFetchCommand, whatsthis);
+  enableFetchCommand(false);
 
   // connect all the signals
   connect(mailboxes, SIGNAL(selectionChanged(QListViewItem *)),
@@ -1714,7 +1715,7 @@ KBiffAboutTab::KBiffAboutTab(QWidget *parent)
   logo_font.setBold(true);
 
   KURLLabel *logo_label = new KURLLabel(this);
-  logo_label->setURL("http://www.granroth.org/kbiff");
+  logo_label->setURL("http://kbiff.granroth.org");
   logo_label->setFont(logo_font);
   logo_label->setText("KBiff");
   logo_label->setUnderline(false);
@@ -1724,7 +1725,7 @@ KBiffAboutTab::KBiffAboutTab(QWidget *parent)
                       SLOT(homepage(const QString&)));
 
   QLabel *version_label = new QLabel(this);
-  version_label->setText(QString("Version %1\n\nCopyright (C) 1998-2001\nKurt Granroth").arg("3.5"));
+  version_label->setText(QString("Version %1\n\nCopyright (C) 1998-2001\nKurt Granroth").arg("3.5.4"));
 
   KURLLabel *email_label = new KURLLabel(this);
   email_label->setText("granroth@kde.org");
